import argparse
import importlib
import sys

from .services import services


class CustomArgumentParser(argparse.ArgumentParser):
    def error(self, message):
        self.print_usage(sys.stderr)
        self.exit(2, f"aaws: error: {message}\n")


def load_module(collection, action):
    try:
        module_name = f"avtomat_aws.cli.{collection}.{action}"
        return importlib.import_module(module_name)
    except ImportError:
        sys.stderr.write(f"Error: Unable to load module for {collection} {action}\n")
        sys.exit(1)


def set_parser():
    parser = CustomArgumentParser(description="Avtomat AWS CLI")
    subparsers = parser.add_subparsers(dest="collection", required=True)

    for collection, actions in services.items():
        service_parser = subparsers.add_parser(collection)
        service_subparsers = service_parser.add_subparsers(dest="action", required=True)

        for action in actions:
            module = load_module(collection, action)
            cmd_description = getattr(
                module, "ACTION_DESCRIPTION", "No description available"
            )
            cmd_parser = service_subparsers.add_parser(
                action, description=cmd_description
            )

            # Global arguments
            cmd_parser.add_argument(
                "--region",
                help="Region for operation. Leave blank for session default.",
                required=False,
            )
            cmd_parser.add_argument(
                "--debug", action="store_true", help="Increase log verbosity."
            )
            cmd_parser.add_argument(
                "--silent", action="store_true", help="Decrease log verbosity."
            )

            # Action specific arguments
            module.add_cli_arguments(cmd_parser)
            cmd_parser.set_defaults(func=module.cli)

    return parser


def main():
    parser = set_parser()
    args = parser.parse_args()
    args.func(args)


if __name__ == "__main__":
    main()



from avtomat_aws.services.s3 import discover_objects

ACTION_DESCRIPTION = "Discover objects in an S3 bucket."


def add_cli_arguments(parser):
    """Argument parsing"""

    required = parser.add_argument_group("required arguments")

    required.add_argument(
        "--bucket", help="Name of the S3 bucket to discover objects in.", required=True
    )
    parser.add_argument("--prefix", help="Prefix to filter objects by.", required=False)
    parser.add_argument(
        "--modified_before",
        help="Get objects last modified before date (YYYY/MM//DD).",
        required=False,
    )
    parser.add_argument(
        "--modified_after",
        help="Get objects last modified after date (YYYY/MM//DD).",
        required=False,
    )
    parser.add_argument(
        "--name_only",
        action="store_true",
        help="Return only the object name, not the entire path.",
        required=False,
    )


def cli(args):
    """Command-line interface function"""

    inputs = vars(args)

    try:
        result = discover_objects(**inputs)
        for item in result:
            print(item)
    except Exception as e:
        print(f"Action failed - {e}")
        exit(1)
